%%This is a part of multi-layered random forest (MLRF) framework

%Copyright (C) 2014 Alina Kuznetsova

%This program is free software: you can redistribute it and/or modify
%it under the terms of the GNU Lesser General Public License as published by
%the Free Software Foundation, either version 2.1 of the License, or
%(at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU Lesser General Public License for more details.

%You should have received a copy of the GNU General Public License
%along with this program.  If not, see <http://www.gnu.org/licenses/>.

%Author: Alina Kuznetsova

%% Computes confusion matrix
%Inputs:
%   clustering,y - Nx1 or 1xN, vectors of values to compute confusion between
%Output:
%   result - confunsion matrix of size clrange x yrange

function [result,clrange,yrange] = confusionMatrix(clustering,y)

    if (~isnumeric(clustering) || ~isnumeric(y))
        warning('numeric values are expected')
    end

    if (size(y,1) > size(y,2))
        %column
        y = y';
    end
    
%    if (~exist('grid','var'))
        warning('computing the grid variable based on the 1..max()');
        disp(size(clustering))
        disp(size(y))
        yrange = min(y):max(y);
        clrange = min(clustering):max(clustering);
%    end
    
%     if (sum(~ismember(clustering,grid))>0)
%         warning('clustering contains values not specified in grid');
%     end
        
%     if (sum(~ismember(y,grid))>0)
%         warning('y contains values not specified in grid');
%     end
    
%     if (sum(sort(grid)~=grid)>0)
%         warning('values in grid are not sorted in the correct order')
%     end
    
    result = zeros(length(clrange),length(yrange));
    
    for i=1:length(clrange)
        if(sum(clustering==clrange(i))>0)
            n = histc(y(clustering==clrange(i)),yrange);
            result(i,:) = n;
        end
    end

end