%only for binary trees - Y should take two values

function newT = pruneTree(T,params)

    if exist('params','var')
        if ~isfield(params,'c')
            params.c = 0.1; %c bigger -> a bigger -> more nodes pruned
        end
        if ~isfield(params,'delta')
            params.delta = 1; %delta bigger -> a smaller -> less nodes pruned (more complicated relation)
        end
    else
        params.c = 0.1;
        params.delta = 1;
    end

%check if data indices field exists
    if (length(T.classes)>2)
        warning('works for two classes only');
        newT = T;    
        return;
    end
    
    ndIds = length(T.weakModels);
    depth = T.depth; %leaf level is depth+1

    %count misclassified samples according to the rule:
    %leaf have a label of majority
    %error is consequently the size of minority
    %leafE(i) is the error for the leaf i
    leafE=min(T.leafdist.*repmat(T.leaftotal,1,2),[],2);  
    %total number of samples
    N = sum(T.leaftotal);
    
    %set subnode-count (subnode is something that actually splits the
    %sample
    for i= (ndIds+1)/2:ndIds
        
        if (T.leaftotal(2*i-ndIds)>0 && T.leaftotal(2*i-ndIds+1)>0)
            T.weakModels{i}.sncount = 1; %there is a split
        else
            T.weakModels{i}.sncount = 0; %no split
        end
        
    end

    for i= ndIds:-1:1
        level = floor(log2(i))+1;%level number
        
        ddepth = depth-level;
        
        if (i< (ndIds+1)/2)
                T.weakModels{i}.sncount = T.weakModels{2*i}.sncount + T.weakModels{2*i+1}.sncount + ...
                (sum(T.weakModels{2*i}.classDist)>0 & sum(T.weakModels{2*i+1}.classDist)>0); %compute the number of subnodes(=splits) reqursively 
        end  
        
        if (T.weakModels{i}.classifierID ~= 0)
            trainE = sum(leafE((2.^ddepth*i-ndIds):(2.^ddepth*(i+1)-1-ndIds)))/sum(T.weakModels{i}.classDist);
            replaceE = min(T.weakModels{i}.classDist)/sum(T.weakModels{i}.classDist); %the smaller number of elements will be misclassified
        
            a = alpha(level,T.weakModels{i}.sncount,sum(T.weakModels{i}.classDist),N,params);
            
            disp(['TrainE: ' num2str(trainE) ' replaceE: ' num2str(replaceE) ' alpha: ' num2str(a)]);
        
            if(trainE + a > replaceE)
                %T1 = T;
                T = pruneNode(T,i); %modify tree
                
                if (0)
                    subplot(2,1,1);
                    cla(gca)
                    showTree(T1);
                    subplot(2,1,2);
                    cla(gca)
                    showTree(T);
                    pause
                end
            end
        end
    end
    
    newT = T;

end

function a = alpha(depth,subnodes,samples,N,params)

    c = params.c; delta = params.delta;

    a = c*sqrt(((depth+subnodes)*log(2*N)+log(N/delta))/samples);
end

%prune the node with the index i
function T = pruneNode(T,i)

    T.weakModels{i}.sncount = 0;
    T.weakModels{i}.nosplit = 1;
    
    level = floor(log2(i))+1;
    ndIds = length(T.weakModels);
    
    delta = T.depth-level-1;
    
    if (i< (ndIds+1)/2)
        T.weakModels{2*i+1}.classDist = T.weakModels{i}.classDist;
    end
    
    rIdx = 2*i+1;
    lIdx = 2*i;
    
    rightLeafIds = (2.^(delta)*(rIdx)-ndIds):(2.^delta*(rIdx+1)-1-ndIds);
    leftLeafIds = (2.^(delta)*(lIdx)-ndIds):(2.^delta*(lIdx+1)-1-ndIds);
    
    %propagate right part (leave only the most right part)
    T.leaftotal(rightLeafIds(1:end-1)) = 0;
    T.leafdist(rightLeafIds(1:end-1),:) = 0;
    T.leaftotal(rightLeafIds(end)) = sum(T.weakModels{i}.classDist);
    T.leafdist(rightLeafIds(end),:) = T.weakModels{i}.classDist/sum(T.weakModels{i}.classDist);

    for d = 1:(delta)
        for k=2^(d-1)*rIdx:2^(d-1)*(rIdx+1)-2
            T.weakModels{k}.classifierID = 0;
        end
        T.weakModels{2^(d-1)*(rIdx+1)-1}.classDist = T.weakModels{i}.classDist;
        T.weakModels{2^(d-1)*(rIdx+1)-1}.nosplit = 1;
        T.weakModels{2^(d-1)*(rIdx+1)-1}.classifierID = 2;
    end

    %remove left part
    T.leaftotal(leftLeafIds) = 0; %no samples in the terminal nodes nodes
    T.leafdist(leftLeafIds,:) = 0;

    for d = 1:(delta)
        for k=2^(d-1)*lIdx:2^(d-1)*(lIdx+1)-1
            T.weakModels{k}.classifierID = 0; %we need to do it for all subnodes.. shit.
        end   
    end

end
